# Namespace package: https://docs.python.org/2/library/pkgutil.html
from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)

__version__ = "0.1.1"
__author__ = "GDS Developers"
__author_email__ = "performance@digital.cabinet-office.gov.uk"
